@media (max-width: 992px) {
  .navbar-header {
    float: none; }

  .navbar-left, .navbar-right {
    float: none !important; }

  .navbar-toggle {
    display: block; }

  .navbar-collapse {
    border-top: 1px solid transparent;
    box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.1); }

  .navbar-fixed-top {
    top: 0;
    border-width: 0 0 1px; }

  .navbar-collapse.collapse {
    display: none !important; }

  .navbar-nav {
    float: none !important;
    margin-top: 7.5px; }

  .navbar-nav > li {
    float: none; }

  .navbar-nav > li > a {
    padding-top: 10px;
    padding-bottom: 10px; }

  .collapse.in {
    display: block !important; } }
/*
 * Off Canvas
 * --------------------------------------------------
 */
@media screen and (max-width: 767px) {
  .row-offcanvas {
    position: relative;
    -webkit-transition: all .25s ease-out;
    -o-transition: all .25s ease-out;
    transition: all .25s ease-out; }

  .row-offcanvas-right {
    right: 0; }

  .row-offcanvas-left {
    left: 0; }

  .row-offcanvas-right
  .sidebar-offcanvas {
    right: -50%;
    /* 6 columns */ }

  .row-offcanvas-left
  .sidebar-offcanvas {
    left: -50%;
    /* 6 columns */ }

  .row-offcanvas-right.active {
    right: 50%;
    /* 6 columns */ }

  .row-offcanvas-left.active {
    left: 50%;
    /* 6 columns */ }

  .sidebar-offcanvas {
    position: absolute;
    top: 0;
    width: 50%;
    /* 6 columns */ } }

/*# sourceMappingURL=admin.css.map */
