
{% import "Juice/macros/base.html" as base with context %}
{% include "Juice/Plugin/Publisher/Page/_include.html" %}

<div class="Publisher-Page">
    <h1>Archive</h1>

    <ul class="archive">
        {% for post in posts %}
            <li>{{ post.published_at | datetime("%B %Y") }}</li>
        {% endfor %}
    </ul>

</div>

