
{% extends "Juice/Plugin/User/Account/_layout.html" %}

{% block login_content %}

    {% call widget.panel( login_title or 'Lost Password') %}

        {% call forms.post(endpoint_namespace % 'lost_password') %}

            {{ forms.hidden("next", value=login_url_next) }}

            {{ forms.input(label="Email", name="email", type="email", data={"validation": "email"}) }}

            {{ forms.button("Retrieve Password", type="submit", class="btn btn-primary btn-lg btn-block") }}

        {% endcall %}

        <hr>

        <a href="{{url_for(endpoint_namespace % 'login')}}">Login</a>

    {% endcall %}

{% endblock %}