@import "mixins";

body.dragging, body.dragging * {
  cursor: move !important;
}

.dragged {
  position: absolute;
  opacity: 0.5;
  z-index: 2000;
}


.sorted_table {
  tr {
    cursor: pointer;
    &.placeholder {
      display: block;
      background: red;
      position: relative;
      margin: 0;
      padding: 0;
      border: none;
      &:before {
        content: "";
        position:absolute;
        width: 0;
        height: 0;
        border: 5px solid transparent;
        margin-top: -5px;
        left: -5px;
        border-right: none;
      }

    }
  }
}


.preview {

  h1 {
    &.main-title {
      font-weight: bold;
      text-align: center;
      font-size: 50px;
    }
  }

  .content {
    @include  responsive-image;
    font-size: 18px;
  }
}


#images-page {

  .modal-image-preview {
    @include  responsive-image;
    margin-bottom: 10px;
  }

  .image-container {
      border: 3px dotted #ccc;
      margin: 10px 0px;
      height: 200px;
      overflow: hidden;
      padding: 4px;
      .image {
        @include  responsive-image;
        padding: 4px;
        img {
          max-height: 150px;
        }
        text-align: center;
      }

      .info {
          text-align: center;

      }
      &:hover {
        background: #ecf0f1;
        .info {
          font-weight: bold;
        }
      }
  }
}

#EditPost {

  .post-header-status {
    margin-bottom: 20px;
  }

  .featured-image-container {
      @include responsive-image;
  }
}


